@@ -1,7 +1,6 @@
 ;;; semantic/db.el --- Semantic tag database manager
 
-;; Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-;;   2009, 2010  Free Software Foundation, Inc.
+;; Copyright (C) 2000-2012 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: tags
@@ -206,7 +205,7 @@ If one doesn't exist, create it."
    )
   "Table used for search results when there is no file or table association.
 Examples include search results from external sources such as from
-Emacs' own symbol table, or from external libraries.")
+Emacs's own symbol table, or from external libraries.")
 
 (defmethod semanticdb-refresh-table ((obj semanticdb-search-results-table) &optional force)
   "If the tag list associated with OBJ is loaded, refresh it.
@@ -228,9 +227,9 @@ it is in Emacs.")
 	  "Non nil if this table needs to be `Saved'.")
    (db-refs :initform nil
 	    :documentation
-	    "List of `semanticdb-table' objects refering to this one.
+	    "List of `semanticdb-table' objects referring to this one.
 These aren't saved, but are instead recalculated after load.
-See the file semanticdb-ref.el for how this slot is used.")
+See the file semantic/db-ref.el for how this slot is used.")
    (pointmax :initarg :pointmax
 	     :initform nil
 	     :documentation "Size of buffer when written to disk.
@@ -442,7 +441,7 @@ Tools needing a per-file cache must subclass this, and then get one as
 needed.  Cache objects are identified in semanticdb by subclass.
 In order to keep your cache up to date, be sure to implement
 `semanticdb-synchronize', and `semanticdb-partial-synchronize'.
-See the file semantic-scope.el for an example."
+See the file semantic/scope.el for an example."
   :abstract t)
 
 (defmethod semanticdb-cache-get ((table semanticdb-abstract-table)
@@ -460,7 +459,7 @@ other than :table."
       (setq cache (cdr cache)))
     (if obj
 	obj ;; Just return it.
-      ;; No object, lets create a new one and return that.
+      ;; No object, let's create a new one and return that.
       (setq obj (funcall desired-class "Cache" :table table))
       (object-add-to-list table 'cache obj)
       obj)))
@@ -493,7 +492,7 @@ Tools needing a database cache must subclass this, and then get one as
 needed.  Cache objects are identified in semanticdb by subclass.
 In order to keep your cache up to date, be sure to implement
 `semanticdb-synchronize', and `semanticdb-partial-synchronize'.
-See the file semantic-scope.el for an example."
+See the file semantic/scope.el for an example."
   :abstract t)
 
 (defmethod semanticdb-cache-get ((db semanticdb-project-database)
@@ -511,7 +510,7 @@ other than :table."
       (setq cache (cdr cache)))
     (if obj
 	obj ;; Just return it.
-      ;; No object, lets create a new one and return that.
+      ;; No object, let's create a new one and return that.
       (setq obj (funcall desired-class "Cache" :db db))
       (object-add-to-list db 'cache obj)
       obj)))
@@ -542,10 +541,25 @@ Optional argument FORCE will force a refresh even if the file in question
 is not in a buffer.  Avoid using FORCE for most uses, as an old cache
 may be sufficient for the general case.  Forced updates can be slow.
 This will call `semantic-fetch-tags' if that file is in memory."
-  (when (or (semanticdb-in-buffer-p obj) force)
+  (cond
+   ;;
+   ;; Already in a buffer, just do it.
+   ((semanticdb-in-buffer-p obj)
+    (semanticdb-set-buffer obj)
+    (semantic-fetch-tags))
+   ;;
+   ;; Not in a buffer.  Forcing a load.
+   (force
+    ;; Patch from Iain Nicol. --
+    ;; @TODO: I wonder if there is a way to recycle
+    ;;        semanticdb-create-table-for-file-not-in-buffer
     (save-excursion
-      (semanticdb-set-buffer obj)
-      (semantic-fetch-tags))))
+      (let ((buff (semantic-find-file-noselect
+		   (semanticdb-full-filename obj))))
+	(set-buffer buff)
+	(semantic-fetch-tags)
+	;; Kill off the buffer if it didn't exist when we were called.
+	(kill-buffer buff))))))
 
 (defmethod semanticdb-needs-refresh-p ((obj semanticdb-table))
   "Return non-nil of OBJ's tag list is out of date.
@@ -592,7 +606,7 @@ The file associated with OBJ does not need to be in a buffer."
   ;; The lexical table should be good too.
   (when (featurep 'semantic/lex-spp)
     (oset table lexical-table (semantic-lex-spp-save-table)))
-  ;; this implies dirtyness
+  ;; this implies dirtiness
   (semanticdb-set-dirty table)
 
   ;; Synchronize the index
@@ -622,7 +636,7 @@ The file associated with OBJ does not need to be in a buffer."
   (when (featurep 'semantic/lex-spp)
     (oset table lexical-table (semantic-lex-spp-save-table)))
 
-  ;; Incremental parser doesn't mokey around with this.
+  ;; Incremental parser doesn't monkey around with this.
   (oset table unmatched-syntax semantic-unmatched-syntax-cache)
 
   ;; Synchronize the index
@@ -643,7 +657,7 @@ The file associated with OBJ does not need to be in a buffer."
 ;;; SAVE/LOAD
 ;;
 (defmethod semanticdb-save-db ((DB semanticdb-project-database)
-			       &optional supress-questions)
+			       &optional suppress-questions)
   "Cause a database to save itself.
 The database base class does not save itself persistently.
 Subclasses could save themselves to a file, or to a database, or other
@@ -808,12 +822,14 @@ Always append `semanticdb-project-system-databases' if
     (setq root (run-hook-with-args-until-success
 		'semanticdb-project-root-functions
 		dir))
-    ;; Find roots based on strings
-    (while (and roots (not root))
-      (let ((r (file-truename (car roots))))
-	(if (string-match (concat "^" (regexp-quote r)) dir)
-	    (setq root r)))
-      (setq roots (cdr roots)))
+    (if root
+	(setq root (file-truename root))
+      ;; Else, Find roots based on strings
+      (while roots
+	(let ((r (file-truename (car roots))))
+	  (if (string-match (concat "^" (regexp-quote r)) dir)
+	      (setq root r)))
+	(setq roots (cdr roots))))
 
     ;; If no roots are found, use this directory.
     (unless root (setq root dir))
@@ -864,7 +880,7 @@ If file does not have tags available, and DONTLOAD is nil,
 then load the tags for FILE, and create a new table object for it.
 DONTLOAD does not affect the creation of new database objects."
   ;; (message "Object Translate: %s" file)
-  (when (file-exists-p file)
+  (when (and file (file-exists-p file))
     (let* ((default-directory (file-name-directory file))
 	   (tab (semanticdb-file-table-object-from-hash file))
 	   (fullfile nil))
@@ -925,7 +941,7 @@ DONTLOAD does not affect the creation of new database objects."
 	  (setq fullfile (file-truename file))
 	  )
 
-	;; If we have a table, but no fullfile, that's ok.  Lets get the filename
+	;; If we have a table, but no fullfile, that's ok.  Let's get the filename
 	;; from the table which is pre-truenamed.
 	(when (and (not fullfile) tab)
 	  (setq fullfile (semanticdb-full-filename tab)))
@@ -1023,5 +1039,4 @@ If file does not have tags available, then load the file, and create them."
 ;; generated-autoload-load-name: "semantic/db"
 ;; End:
 
-;; arch-tag: d9f75280-737d-494f-9f70-09a649d27433
 ;;; semantic/db.el ends here
