@@ -1,6 +1,5 @@
 /* The emacs frame widget public header file.
-   Copyright (C) 1993, 2001, 2002, 2003, 2004, 2005,
-                 2006, 2007, 2008, 2009, 2010  Free Software Foundation, Inc.
+   Copyright (C) 1993, 2001-2012  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -93,11 +92,9 @@ extern WidgetClass emacsFrameClass;
 
 extern struct _DisplayContext* display_context;
 
-/* Special entrypoints */
-void EmacsFrameSetCharSize P_ ((Widget, int, int));
-void widget_store_internal_border P_ ((Widget widget));
+/* Special entry points */
+void EmacsFrameSetCharSize (Widget, int, int);
+void widget_store_internal_border (Widget widget);
+void widget_update_wm_size_hints (Widget widget);
 
 #endif /* _EmacsFrame_h */
-
-/* arch-tag: 98be17cc-8878-4701-abfa-66f1c04e9cb7
-   (do not change this comment) */
